import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useContext, useState, useRef, useCallback } from 'react';
import { useClassName } from '../hooks/className';
import { useFireEvent } from '../events';
import { useRequest } from '../tools';
import { LocationContext } from '../hooks/locationContext';
import { AnyCompList } from './index';
import { ButtonComp } from './button';
export const FormComp = (props) => {
    const formRef = useRef(null);
    const { formFields, initial, submitUrl, method, footer, displayMode, submitOnChange } = props;
    // mostly equivalent to `<input disabled`
    const [locked, setLocked] = useState(false);
    const [fieldErrors, setFieldErrors] = useState({});
    const [error, setError] = useState(null);
    const { fireEvent } = useFireEvent();
    const request = useRequest();
    const { goto } = useContext(LocationContext);
    const submit = useCallback(async (formData) => {
        setLocked(true);
        setError(null);
        setFieldErrors({});
        if (method === 'GOTO') {
            // this seems to work in common cases, but typescript doesn't like it
            const query = new URLSearchParams(formData);
            for (const [k, v] of query.entries()) {
                if (v === '') {
                    query.delete(k);
                }
            }
            const queryStr = query.toString();
            goto(queryStr === '' ? submitUrl : `${submitUrl}?${queryStr}`);
            setLocked(false);
            return;
        }
        const requestArgs = { url: submitUrl, expectedStatus: [200, 422] };
        if (method === 'GET') {
            // as above with URLSearchParams
            requestArgs.query = new URLSearchParams(formData);
        }
        else {
            requestArgs.formData = formData;
        }
        const [status, data] = await request(requestArgs);
        if (status === 200) {
            if (data.type !== 'FormResponse') {
                throw new Error(`Expected FormResponse, got ${JSON.stringify(data)}`);
            }
            const { event } = data;
            fireEvent(event);
        }
        else {
            // status === 422
            const errorResponse = data;
            const formErrors = errorResponse.detail.form;
            if (formErrors) {
                setFieldErrors(Object.fromEntries(formErrors.map((e) => [locToName(e.loc), e.msg])));
            }
            else {
                console.warn('Non-field error submitting form:', data);
                setError('Error submitting form');
            }
        }
        setLocked(false);
    }, [goto, method, request, submitUrl, fireEvent]);
    const onSubmit = async (e) => {
        e.preventDefault();
        const formData = new FormData(e.currentTarget);
        await submit(formData);
    };
    const onChange = useCallback(() => {
        if (submitOnChange && formRef.current) {
            const formData = new FormData(formRef.current);
            submit(formData);
        }
    }, [submitOnChange, submit]);
    const fieldProps = formFields.map((formField) => {
        const f = {
            ...formField,
            error: fieldErrors[formField.name],
            locked,
            displayMode,
            onChange,
        };
        const formInitial = initial && initial[formField.name];
        if (formInitial !== undefined) {
            ;
            f.initial = formInitial;
        }
        return f;
    });
    return (_jsx("div", { className: useClassName(props, { el: 'form-container' }), children: _jsxs("form", { ref: formRef, className: useClassName(props), onSubmit: onSubmit, children: [_jsx(AnyCompList, { propsList: fieldProps }), error ? _jsxs("div", { children: ["Error: ", error] }) : null, _jsx(Footer, { footer: footer })] }) }));
};
const Footer = ({ footer }) => {
    if (footer === false) {
        return null;
    }
    else if (footer === true || typeof footer === 'undefined') {
        return _jsx(ButtonComp, { type: "Button", text: "Submit", htmlType: "submit" });
    }
    else {
        return _jsx(AnyCompList, { propsList: footer });
    }
};
/**
 * Should match `loc_to_name` in python so errors can be connected to the correct field
 */
function locToName(loc) {
    if (loc.some((v) => typeof v === 'string' && v.includes('.'))) {
        return JSON.stringify(loc);
    }
    else if (typeof loc[0] === 'string' && loc[0].startsWith('[')) {
        return JSON.stringify(loc);
    }
    else {
        return loc.join('.');
    }
}
